
#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// Copyright © 2011, Integrated Trading Technologies   itt@integratedtradingtech.com

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Chande's Intraday Momentum Index
    /// </summary>
    [Description("Chande's Intraday Momentum Index")]
    public class IMI : Indicator
    {
        #region Variables        
        private int length = 13;
		private int avgLength = 8;
		private bool showSignal = false;        
		private DataSeries Up;
		private DataSeries Down;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Orange), PlotStyle.Line, "IMI_"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Green), PlotStyle.Line, "Signal"));
			Add(new Line(Color.FromKnownColor(KnownColor.DarkOliveGreen), 70, "Upper"));
			Add(new Line(Color.FromKnownColor(KnownColor.DarkOliveGreen), 30, "Lower"));
            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= false;
			Up = new DataSeries(this);
			Down = new DataSeries(this);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {			
			if (CurrentBar == 0)
			{
				Down.Set(0);
				Up.Set(0);
				return;
			}
			
			Up.Set(Close[0] > Open[0] ? Close[0] - Open[0] : 0);
			Down.Set(Close[0] < Open[0] ? Open[0] - Close[0] : 0);			
			
            IMI_.Set(SUM(Up, Length)[0] / (SUM(Up, Length)[0] + SUM(Down, Length)[0]) * 100);
			if (ShowSignal && CurrentBar > AvgLength) Signal.Set(SMA(IMI_, AvgLength)[0]);
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries IMI_
        {
            get { return Values[0]; }
        }

		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
		[XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
		public DataSeries Signal
		{
			get { return Values[1]; }
		}

        [Description("Length of lookback period")]
        [Category("Parameters")]
        public int Length
        {
            get { return length; }
            set { length = Math.Max(1, value); }
        }

		[Description("Length of signal line")]
		[Category("Parameters")]
		public int AvgLength
		{
			get { return avgLength; }
			set { avgLength = Math.Max(1, value); }
		}

		[Description("Turns the signal line on/off")]
		[Category("Parameters")]
		public bool ShowSignal
		{
			get { return showSignal; }
			set { showSignal = value; }
		}
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private IMI[] cacheIMI = null;

        private static IMI checkIMI = new IMI();

        /// <summary>
        /// Chande's Intraday Momentum Index
        /// </summary>
        /// <returns></returns>
        public IMI IMI(int length)
        {
            return IMI(Input, length);
        }

        /// <summary>
        /// Chande's Intraday Momentum Index
        /// </summary>
        /// <returns></returns>
        public IMI IMI(Data.IDataSeries input, int length)
        {
            if (cacheIMI != null)
                for (int idx = 0; idx < cacheIMI.Length; idx++)
                    if (cacheIMI[idx].Length == length && cacheIMI[idx].EqualsInput(input))
                        return cacheIMI[idx];

            lock (checkIMI)
            {
                checkIMI.Length = length;
                length = checkIMI.Length;

                if (cacheIMI != null)
                    for (int idx = 0; idx < cacheIMI.Length; idx++)
                        if (cacheIMI[idx].Length == length && cacheIMI[idx].EqualsInput(input))
                            return cacheIMI[idx];

                IMI indicator = new IMI();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Length = length;
                Indicators.Add(indicator);
                indicator.SetUp();

                IMI[] tmp = new IMI[cacheIMI == null ? 1 : cacheIMI.Length + 1];
                if (cacheIMI != null)
                    cacheIMI.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheIMI = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Chande's Intraday Momentum Index
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.IMI IMI(int length)
        {
            return _indicator.IMI(Input, length);
        }

        /// <summary>
        /// Chande's Intraday Momentum Index
        /// </summary>
        /// <returns></returns>
        public Indicator.IMI IMI(Data.IDataSeries input, int length)
        {
            return _indicator.IMI(input, length);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Chande's Intraday Momentum Index
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.IMI IMI(int length)
        {
            return _indicator.IMI(Input, length);
        }

        /// <summary>
        /// Chande's Intraday Momentum Index
        /// </summary>
        /// <returns></returns>
        public Indicator.IMI IMI(Data.IDataSeries input, int length)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.IMI(input, length);
        }
    }
}
#endregion
